@@ -92,7 +92,7 @@ class GTKInterface(UserInterface):
         self.IMAGE_HEAD_BACKGROUND = posixpath.join(data_path,
             "checkbox-gtk-head.png")
         self.FONT = "Ubuntu"
-        self.TEXT =_("System Testing")
+        self.TEXT = title
 
         image_head=self._get_widget("image_head")
         try:
@@ -116,7 +116,7 @@ class GTKInterface(UserInterface):
         for radio_button, value in map.items():
             if self._get_widget(radio_button).get_active():
                 return value
-        raise Exception, "Failed to map radio_button."
+        raise Exception("Failed to map radio_button.")
 
     def _get_label(self, name):
         widget = self._get_widget(name)
@@ -286,7 +286,9 @@ class GTKInterface(UserInterface):
         self._set_button("button_previous", previous_state)
         self._set_button("button_next", next_state)
 
-        return self._get_text("entry")
+        entered_value = self._get_text("entry")
+        self._set_text("entry","")
+        return entered_value
 
     @GTKHack
     def show_check(self, text, options=[], default=[]):
@@ -392,12 +394,13 @@ class GTKInterface(UserInterface):
         treeview.show()
 
         def set_options(options, default, parent=None):
-            keys = sorted(options.keys())
-            values = [options[k] for k in keys]
-            for text, child in zip(keys, values):
-                active = text in default
-                iter = treestore.append(parent, [text, active])
-                set_options(child, default.get(text, {}), iter)
+            if isinstance(options, dict):
+                keys = sorted(options.keys())
+                values = [options[k] for k in keys]
+                for text, child in zip(keys, values):
+                    active = text in default
+                    iter = treestore.append(parent, [text, active])
+                    set_options(child, default.get(text, {}), iter)
 
         set_options(options, default)
 
@@ -537,6 +540,8 @@ class GTKInterface(UserInterface):
         test["data"] = self._get_text_view("text_view_comment")
         test["status"] = self._get_radio_button(BUTTON_TO_STATUS)
 
+        self._set_main_title()
+
     def show_info(self, text, options=[], default=None):
         message_dialog = Gtk.MessageDialog(parent=self._dialog,
             type=Gtk.MessageType.INFO,
@@ -544,10 +549,15 @@ class GTKInterface(UserInterface):
             message_format=text)
         message_dialog.set_modal(True)
         message_dialog.set_title(_("Info"))
+        message_dialog.connect("realize", lambda x: \
+            x.get_window().set_functions(Gdk.WMFunction.MOVE))
 
+        if default:
+            # We have a default, move it to the end of the button list
+            options.remove(default)
+            options.append(default)
         for index, option in enumerate(options):
-            button = getattr(Gtk, "STOCK_%s" % option.upper())
-            message_dialog.add_buttons(button, index)
+            message_dialog.add_button(option, index)
 
         self._run_dialog(message_dialog)
         message_dialog.hide()
